<DATE>	Alexandre Oliva  <oliva@dcc.unicamp.br>

	* config/sparc/common.h: (sysdepCallMethod) total rewrite,
	in order to correctly handle jlongs and jdoubles arguments and
	return values, as well as long (>6 words) argument lists.
	* kaffe/kaffevm/support.c: (callMethodV) assume jboolean,
	jshort, jbyte and jchar are all promoted to jint, just like
	most of kaffe.def does.  Fixed handling of jdouble and jlong.

Index: config/sparc/common.h
===================================================================
RCS file: /home/msc/oliva/src/.cvs/kaffe/config/sparc/common.h,v
retrieving revision 1.1.1.1
diff -u -r1.1.1.1 common.h
--- common.h	1998/03/30 23:11:01	1.1.1.1
+++ common.h	1998/05/01 00:21:43
@@ -12,25 +12,77 @@
 #ifndef __sparc_common_h
 #define __sparc_common_h
 
-#define	sysdepCallMethod(CALL)						\
-	asm volatile ("							\n\
-		cmp %2,0						\n\
-		be 1f							\n\
-		nop							\n\
-		ld [%2],%%o0						\n\
-		ld [%2+8],%%o1						\n\
-		ld [%2+16],%%o2						\n\
-		ld [%2+24],%%o3						\n\
-		ld [%2+32],%%o4						\n\
-		ld [%2+40],%%o5						\n\
-	1:								\n\
-		call %1,0						\n\
-		nop							\n\
-		st %%o0,[%0]						\n\
-	" :								\
-	  : "r" ((CALL)->ret),						\
-	    "r" ((CALL)->function),					\
-	    "r" ((CALL)->args)						\
-	  : "g1" , "g2", "g3", "g4", "o0", "o1", "o2", "o3", "o4", "o5", "o7" )
+#define	sysdepCallMethod(CALL) do {					\
+  /* allocates variable arrays one word past the minimum stack frame */ \
+  int extraargs[((CALL)->nrargs>6)?((CALL)->nrargs-6):0];		\
+  switch((CALL)->nrargs) {						\
+    register int o0 asm("o0");						\
+    register int o1 asm("o1");						\
+    register int o2 asm("o2");						\
+    register int o3 asm("o3");						\
+    register int o4 asm("o4");						\
+    register int o5 asm("o5");						\
+  default:								\
+    {									\
+      int *args = extraargs-1;						\
+      int argidx = 6; 							\
+      for(; argidx < (CALL)->nrargs; ++argidx, ++args) {		\
+	if ((CALL)->callsize[argidx])					\
+	  *args = (CALL)->args[argidx].i;				\
+	else								\
+	  *args = (CALL)->args[argidx-1].j;				\
+      }									\
+    }									\
+  case 6:								\
+    if ((CALL)->callsize[5])						\
+      o5 = (CALL)->args[5].i;						\
+    else								\
+      o5 = (CALL)->args[4].j;						\
+  case 5:								\
+    if ((CALL)->callsize[4])						\
+      o4 = (CALL)->args[4].i;						\
+    else								\
+      o4 = (CALL)->args[3].j;						\
+  case 4:								\
+    if ((CALL)->callsize[3])						\
+      o3 = (CALL)->args[3].i;						\
+    else								\
+      o3 = (CALL)->args[2].j;						\
+  case 3:								\
+    if ((CALL)->callsize[2])						\
+      o2 = (CALL)->args[2].i;						\
+    else								\
+      o2 = (CALL)->args[1].j;						\
+  case 2:								\
+    if ((CALL)->callsize[1])						\
+      o1 = (CALL)->args[1].i;						\
+    else								\
+      o1 = (CALL)->args[0].j;						\
+  case 1:								\
+  case 0:								\
+    asm ("call %2,0\n							\
+          ld %3, %0\n"							\
+	: "=r" (o0), "=r" (o1)						\
+	: "r" ((CALL)->function),					\
+	  "m" ((CALL)->args?(CALL)->args[0].i:0),			\
+	  "0" (o0), "1" (o1), "r" (o2), "r" (o3), "r" (o4), "r" (o5)	\
+	: "g1", "g2", "g3", "g4", "o7", "cc"				\
+	);								\
+    switch((CALL)->retsize) {						\
+    case 2:								\
+      (CALL)->ret->j = (((long long)o0) << 32) | o1;			\
+      break;								\
+    case 1:								\
+      (CALL)->ret->i = o0;						\
+      break;								\
+    case 0:								\
+      break;								\
+    default:								\
+      ABORT();								\
+      break;								\
+    }									\
+    break;								\
+  }									\
+} while (0)								\
 
 #endif
Index: kaffe/kaffevm/support.c
===================================================================
RCS file: /home/msc/oliva/src/.cvs/kaffe/kaffe/kaffevm/support.c,v
retrieving revision 1.1.1.7
diff -u -r1.1.1.7 support.c
--- support.c	1998/04/21 11:40:12	1.1.1.7
+++ support.c	1998/05/01 00:21:43
@@ -362,24 +362,12 @@
 		call.calltype[i] = *sig;
 		switch (*sig) {
 		case 'I':
-			call.callsize[i] = 1;
-			in[i].i = va_arg(args, jint);
-			break;
 		case 'Z':
-			call.callsize[i] = 1;
-			in[i].z = va_arg(args, jboolean);
-			break;
 		case 'S':
-			call.callsize[i] = 1;
-			in[i].s = va_arg(args, jshort);
-			break;
 		case 'B':
-			call.callsize[i] = 1;
-			in[i].b = va_arg(args, jbyte);
-			break;
 		case 'C':
 			call.callsize[i] = 1;
-			in[i].c = va_arg(args, jchar);
+			in[i].i = va_arg(args, jint);
 			break;
 		case 'F':
 			call.callsize[i] = 1;
@@ -388,10 +376,14 @@
 		case 'D':
 			call.callsize[i] = 2;
 			in[i].d = va_arg(args, jdouble);
+			++i; s += 2;
+			call.callsize[i] = 0;
 			break;
 		case 'J':
 			call.callsize[i] = 2;
 			in[i].j = va_arg(args, jlong);
+			++i; s += 2;
+			call.callsize[i] = 0;
 			break;
 		case '[':
 			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
